#include "tree.h"
#include "util.h"

void Xobject(struct OMObjectNode *obj);
void Xinteger(struct OMIntegerNode *obj);
void Xsymbol(struct OMSymbolNode *obj);
void Xvariable(struct OMVariableNode *obj);
void Xfloatingp(struct OMFloatingPNode *obj);
void Xstring(struct OMStringNode *obj);
void Xwcstring(struct OMWCStringNode *obj);
void Xbytearray(struct OMByteArrayNode *obj);
void Xapplication(struct OMApplicationNode *obj);
void Xbinding(struct OMBindingNode *obj);
void Xattribution(struct OMAttributionNode *obj);
void Xerror(struct OMErrorNode *obj);
void Xatp(struct OMAtpNode *obj);

void Xtree(struct OMObjectNode *tree)
{
	Xobject(tree);
}

void Xobject(struct OMObjectNode *object)
{
	switch( object->type ) {
	case OMInteger:
		Xinteger(object->node.integer);
		break;
	case OMSymbol:
		Xsymbol(object->node.symbol);
		break;
	case OMVariable:
		Xvariable(object->node.variable);
		break;
	case OMFloatingP:
		Xfloatingp(object->node.floatingp);
		break;
	case OMString:
		Xstring(object->node.string);
		break;
	case OMWCString:
		Xwcstring(object->node.wcstring);
		break;
	case OMByteArray:
		Xbytearray(object->node.bytearray);
		break;
	case OMApplication:
		Xapplication(object->node.application);
		break;
	case OMBinding:
		Xbinding(object->node.binding);
		break;
	case OMAttribution:
		Xattribution(object->node.attribution);
		break;
	case OMError:
		Xerror(object->node.error);
		break;
	default:
		die("Unknown object type %d.\n",object->type);
		break;
	}
}


void Xinteger(struct OMIntegerNode *obj)
{
	obj->value;
}


void Xsymbol(struct OMSymbolNode *obj)
{
	obj->name;
	obj->cd;
}


void Xvariable(struct OMVariableNode *obj)
{
	obj->varname;
}


void Xfloatingp(struct OMFloatingPNode *obj)
{
	obj->value;
}


void Xstring(struct OMStringNode *obj)
{
	obj->text;
}


void Xwcstring(struct OMWCStringNode *obj)
{
	obj->wctext;
}


void Xbytearray(struct OMByteArrayNode *obj)
{
	obj->array;
}


void Xapplication(struct OMApplicationNode *obj)
{
	int n;

	obj->head;

	for( n = 0; (obj->args)[n] != NULL; n++ ) {
		Xobject((obj->args)[n]);
	}
}


void Xbinding(struct OMBindingNode *obj)
{
	int n;

	Xobject(obj->binder);

	for( n = 0; (obj->vars)[n] != NULL; n++ ) {
		Xobject((obj->vars)[n]);
	}

	Xobject(obj->body);
}


void Xattribution(struct OMAttributionNode *obj)
{
	int n;

	for( n = 0; (obj->atps)[n] != NULL; n++ ) {
		Xatp((obj->atps)[n]);
	}

	Xobject(obj->object);
}


void Xerror(struct OMErrorNode *obj)
{
	int n;

	Xsymbol(obj->symbol);

	for( n = 0; (obj->args)[n] != NULL; n++ ) {
		Xobject((obj->args)[n]);
	}
}


void Xatp(struct OMAtpNode *obj)
{
	Xsymbol(obj->attribute);
	Xobject(obj->value);
}
