#include "tree.h"
#include "util.h"
#include "printtree.h"
#include <stdio.h>

void printobject(FILE *fp,struct OMObjectNode *obj);
void printinteger(FILE *fp,struct OMIntegerNode *obj);
void printsymbol(FILE *fp,struct OMSymbolNode *obj);
void printvariable(FILE *fp,struct OMVariableNode *obj);
void printfloatingp(FILE *fp,struct OMFloatingPNode *obj);
void printstring(FILE *fp,struct OMStringNode *obj);
void printwcstring(FILE *fp,struct OMWCStringNode *obj);
void printbytearray(FILE *fp,struct OMByteArrayNode *obj);
void printapplication(FILE *fp,struct OMApplicationNode *obj);
void printbinding(FILE *fp,struct OMBindingNode *obj);
void printattribution(FILE *fp,struct OMAttributionNode *obj);
void printerror(FILE *fp,struct OMErrorNode *obj);
void printatp(FILE *fp,struct OMAtpNode *obj);

void printtree(FILE *fp,struct OMObjectNode *tree)
{
	printobject(fp,tree);
}

void printobject(FILE *fp,struct OMObjectNode *object)
{
	switch( object->type ) {
	case OMInteger:
		printinteger(fp,object->node);
		break;
	case OMSymbol:
		printsymbol(fp,object->node);
		break;
	case OMVariable:
		printvariable(fp,object->node);
		break;
	case OMFloatingP:
		printfloatingp(fp,object->node);
		break;
	case OMString:
		printstring(fp,object->node);
		break;
	case OMWCString:
		printwcstring(fp,object->node);
		break;
	case OMByteArray:
		printbytearray(fp,object->node);
		break;
	case OMApplication:
		printapplication(fp,object->node);
		break;
	case OMBinding:
		printbinding(fp,object->node);
		break;
	case OMAttribution:
		printattribution(fp,object->node);
		break;
	case OMError:
		printerror(fp,object->node);
		break;
	default:
		die("Unknown object type (%d) in printobject!\n",object->type);
		break;
	}
}


void printinteger(FILE *fp,struct OMIntegerNode *obj)
{
	fprintf(fp,"{int value=%s}\n",obj->value);
}


void printsymbol(FILE *fp,struct OMSymbolNode *obj)
{
	fprintf(fp,"{symbol name=%s cd=%s}\n",obj->name,obj->cd);
}


void printvariable(FILE *fp,struct OMVariableNode *obj)
{
	fprintf(fp,"{var name=%s}\n",obj->varname);
}


void printfloatingp(FILE *fp,struct OMFloatingPNode *obj)
{
	fprintf(fp,"{float value=%s}\n",obj->value);
}


void printstring(FILE *fp,struct OMStringNode *obj)
{
	fprintf(fp,"{string value=%s}\n",obj->text);
}


void printwcstring(FILE *fp,struct OMWCStringNode *obj)
{
	int n;

	fprintf(fp,"{wcstring value=");
	for( n = 0 ; (obj->wctext)[n] != 0 ; n++ )
		fprintf(fp,"%04d",(obj->wctext)[n]);
	fprintf(fp,"}\n");
}


void printbytearray(FILE *fp,struct OMByteArrayNode *obj)
{
	int n;

	fprintf(fp,"{bytearray value=0x");
	for( n = 0 ; n < obj->len ; n++ )
		fprintf(fp,"%02d",(obj->array)[n]);
	fprintf(fp,"}\n");
}


void printapplication(FILE *fp,struct OMApplicationNode *obj)
{
	int n;

	fprintf(fp,"{application head=");
	printobject(fp,obj->head);
	fprintf(fp,"args=(");
	for( n = 0; (obj->args)[n] != NULL; n++ ) {
		printobject(fp,(obj->args)[n]);
	}
	fprintf(fp,")}\n");
}


void printbinding(FILE *fp,struct OMBindingNode *obj)
{
	int n;

	fprintf(fp,"{binding binder=");
	printobject(fp,obj->binder);
	fprintf(fp,"vars=(");
	for( n = 0; (obj->vars)[n] != NULL; n++ ) {
		printobject(fp,(obj->vars)[n]);
	}
	fprintf(fp,") body=");
	printobject(fp,obj->body);
	fprintf(fp,"}\n");
}


void printattribution(FILE *fp,struct OMAttributionNode *obj)
{
	int n;

	fprintf(fp,"{attribution object=");
	printobject(fp,obj->object);
	fprintf(fp,"attributes=(");
	for( n = 0; (obj->atps)[n] != NULL; n++ ) {
		printatp(fp,(obj->atps)[n]);
	}
	fprintf(fp,")}\n");
}


void printerror(FILE *fp,struct OMErrorNode *obj)
{
	int n;

	fprintf(fp,"{error symbol=");
	printsymbol(fp,obj->symbol);
	fprintf(fp,"args=(");
	for( n = 0; (obj->args)[n] != NULL; n++ ) {
		printobject(fp,(obj->args)[n]);
	}
	fprintf(fp,")}\n");
}


void printatp(FILE *fp,struct OMAtpNode *obj)
{
	fprintf(fp,"[attribute=");
	printsymbol(fp,obj->attribute);
	fprintf(fp,",value=");
	printobject(fp,obj->value);
}
