#ifndef _TREE_H_
#define _TREE_H_

/*
 * These are the types an OpenMath object can have.
 */

enum OMObjectType {
	OMInteger,
	OMSymbol,
	OMVariable,
	OMFloatingP,
	OMString,
	OMWCString,
	OMByteArray,
	OMApplication,
	OMBinding,
	OMAttribution,
	OMError
};

/*
 * This is the data we remember for each type.
 */

struct OMIntegerNode {
	char *value;			/* String reresenting int value */
};

struct OMSymbolNode {
	char *name;			/* Symbol name */
	char *cd;			/* Content Dictionary */
};

struct OMVariableNode {
	char *varname;			/* Variable name */
	char *unique_name;		/* Unique version of name */
};

struct OMFloatingPNode {
	char *value;			/* String representing float value */
};

struct OMStringNode {
	char *text;			/* Unencoded string */
};

struct OMWCStringNode {
	unsigned short *wctext;		/* Unencoded wide character string */
};

struct OMByteArrayNode {
	char *array;			/* Bytearray in base64 encoding */
	int len;
};

struct OMApplicationNode {
	struct OMObjectNode *head;	/* What we're applying */
	struct OMObjectNode **args;	/* List of what we're applying it to */
};

struct OMBindingNode {
	struct OMObjectNode *binder;	/* What binder we're using */
	struct OMObjectNode **vars;	/* List of variables we're binding */
	struct OMObjectNode *body;	/* What expression is to be bound */
};

struct OMAttributionNode {
	struct OMAtpNode **atps;	/* List of attributes */
	struct OMObjectNode *object;	/* Object we're attributing */
};

struct OMErrorNode {
	struct OMSymbolNode *symbol;	/* The error we encountered */
	struct OMObjectNode **args;	/* The objects it applied to */
};

/*
 * This is a helper type for variables and attributes.
 */
struct OMAtpNode {
	struct OMSymbolNode *attribute;
	struct OMObjectNode *value;
};

/*
 * This is the structure we use when we don't know what type to expect.
 */
struct OMObjectNode {
	enum OMObjectType type;	/* What sort of node are we? */
	void *node;
};

#endif /* _TREE_H_ */
