#
# Functions related to binding of variables.
#

# Returns true if variable is bound.

sub is_bound {
	my ($name) = @_;

	return $bound_var{$name} || $weak_bound_var{$name};
}

# Returns what format is bound to the variable

sub bound_format {
	my ($format) = @_;

	if( $bound_var{$format} ) {
		return ${$bound_format{$format}}[0];
	} elsif ($weak_bound_var{$format}) {
		return $weak_bound_format{$format};
	} else {
		return undef;
	}
}

# Mark a new variable as bound and store the formatting for it.

sub pushbind {
	my ($node) = @_;
	my ($name) = $node->{'name'};
	my ($format);

	$bound_var{$name}++;
	unshift(@{$bound_format{$name}},undef);
	$format = ${$bound_format{$name}}[0] = &format_type($node);
	$format_inuse{$format->{'fmt'}} = 1;
}

# Unmark the variable as bound.

sub popbind {
	my ($node) = @_;
	my ($name) = $node->{'name'};
	my $format;

	$bound_var{$name}--;
	$format = shift(@{$bound_format{$name}});
	$format_inuse{$format->{'fmt'}} = 0;
}

# Bind a format to a variable.

sub bindformat {
	my ($name,$format) = @_;

	if( $bound_var{$name} ) {
		${$bound_format{$name}}[0] = $format;
	} else {
		$weak_bound_var{$name} = 1;
		$weak_bound_format{$name} = $format;
	}

	$format_inuse{$format->{'fmt'}} = 1;
}

# Check if a given format is in use.

sub format_used {
	my ($fmt) = @_;

	return $format_inuse{$fmt};
}

1;
