#

$thiscd = 'arith1';

$cdformatter{$thiscd} = {
	'abs' => {
		'symbol'      => \&tex_arith1_abs_symbol,
		'application' => \&tex_arith1_abs_application,
		},
	'conjugate' => {
		'symbol'      => \&tex_arith1_conjugate_symbol,
		'application' => \&tex_arith1_conjugate_application,
		},
	'divide' => {
		'symbol'      => \&tex_arith1_divide_symbol,
		'application' => \&tex_arith1_divide_application,
		},
	'minus' => {
		'symbol'      => \&tex_arith1_minus_symbol,
		'application' => \&tex_arith1_minus_application,
		},
	'plus' => {
		'symbol'      => \&tex_arith1_plus_symbol,
		'application' => \&tex_arith1_plus_application,
		},
	'power' => {
		'symbol'      => \&tex_arith1_power_symbol,
		'application' => \&tex_arith1_power_application,
		},
	'product' => {
		'symbol'      => \&tex_arith1_product_symbol,
		'application' => \&tex_arith1_product_application,
		},
	'root' => {
		'symbol'      => \&tex_arith1_root_symbol,
		'application' => \&tex_arith1_root_application,
		},
	'sum' => {
		'symbol'      => \&tex_arith1_sum_symbol,
		'application' => \&tex_arith1_sum_application,
		},
	'times' => {
		'symbol'      => \&tex_arith1_times_symbol,
		'application' => \&tex_arith1_times_application,
		},
	'unary_minus' => {
		'symbol'      => \&tex_arith1_unary_minus_symbol,
		'application' => \&tex_arith1_unary_minus_application,
		},
};

sub tex_arith1_abs_symbol {
	return ({'fmt' => '\mbox{abs}', 'sfmt' => '\mbox{abs}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_abs_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 1 ) {
		&format_die("abs with wrong number of args.\n");
	}

	$fmt = '\left| ' . &format_type($args->[0])->{'sfmt'} . ' \right|';

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_conjugate_symbol {
	return ({'fmt' => '\mbox{conjugate}', 'sfmt' => '\mbox{conjugate}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_conjugate_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 1 ) {
		&format_die("conjugate with wrong number of args.\n");
	}

	$fmt = '\overline{' . &format_type($args->[0])->{'sfmt'} . '}';

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_divide_symbol {
	return ({'fmt' => '/', 'sfmt' => '/',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_divide_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("divide with wrong number of args.\n");
	}

	$fmt = '\frac{' . &format_type($args->[0])->{'sfmt'} . '}{' .
		&format_type($args->[1])->{'sfmt'} . '}';

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_minus_symbol {
	return ({'fmt' => '-', 'sfmt' => '-',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_minus_application {
	my ($node) = @_;
        my $args = $node->{'args'};

	if( scalar(@$args) != 2 ) {
		&format_die("minus with wrong number of args.\n");
	}

	return &infix_op($node,'-');
}

sub tex_arith1_plus_symbol {
	return ({'fmt' => '+', 'sfmt' => '+',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_plus_application {
	my ($node) = @_;

	return &infix_op($node,'+');
}

sub tex_arith1_power_symbol {
	return ({'fmt' => '\mbox{pow}', 'sfmt' => '\mbox{pow}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_power_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("power with wrong number of args.\n");
	}

	$fmt = &tex_bracket_sfmt(&format_type($args->[0])) .
		'^' .
		'{' . &format_type($args->[1])->{'sfmt'} . '}';

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_product_symbol {
	return ({'fmt' => '{\prod}', 'sfmt' => '{\prod}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_product_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("product with wrong number of args.\n");
	}

	&format_type($args->[1]);
	if( $args->[1]->{'type'} eq 'binding' ) {
		$fmt = '\prod_{' .
			&format_type($args->[1]->{'vars'}->[0])->{'sfmt'} .
			'=' .
			&format_type($args->[0])->{'sfmt'} . '} ' .
			&format_type($args->[1])->{'sfmt'} ;
	} else {
		$fmt = '\prod_{' . &format_type($args->[0])->{'sfmt'} . '} ' .
			&format_type($args->[1])->{'sfmt'} ;
	}

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_root_symbol {
	return ({'fmt' => '\mbox{root}', 'sfmt' => '\mbox{root}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_root_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("root with wrong number of args.\n");
	}

	$fmt = &tex_bracket_sfmt(&format_type($args->[0])) .
		'^' .
		'\frac{1}{'.&format_type($args->[1])->{'sfmt'} . '}';

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_sum_symbol {
	return ({'fmt' => '{\sum}', 'sfmt' => '{\sum}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_sum_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("sum with wrong number of args.\n");
	}

	&format_type($args->[1]);
	if( $args->[1]->{'type'} eq 'binding' ) {
		$fmt = '\sum_{' .
			&format_type($args->[1]->{'vars'}->[0])->{'sfmt'} .
			'=' .
			&format_type($args->[0])->{'sfmt'} . '} ' .
			&format_type($args->[1])->{'sfmt'};
	} else {
		$fmt = '\sum_{' . &format_type($args->[0])->{'sfmt'} . '} ' .
			&format_type($args->[1])->{'sfmt'};
	}

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_arith1_times_symbol {
	return ({'fmt' => '.', 'sfmt' => '.',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_times_application {
	my ($node) = @_;

	return &infix_op($node,'.');
}

sub tex_arith1_unary_minus_symbol {
	return ({'fmt' => '-', 'sfmt' => '-',
		'lpren' => '', 'rpren' => ''});
}

sub tex_arith1_unary_minus_application {
	my ($node) = @_;
        my $head = $node->{'head'};
        my $args = $node->{'args'};
        my $fmt;

	if( scalar(@$args) != 1 ) {
		&format_die("unary minus with wrong number of args.\n");
	}

	$fmt = '-' . &tex_bracket_sfmt(&format_type($args->[0]));

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

1;
