#

$thiscd = 'calculus1';

$cdformatter{$thiscd} = {
	'defint' => {
		'symbol'      => \&tex_calculus1_defint_symbol,
		'application' => \&tex_calculus1_defint_application,
		},
	'diff' => {
		'symbol'      => \&tex_calculus1_diff_symbol,
		'application' => \&tex_calculus1_diff_application,
		},
	'int' => {
		'symbol'      => \&tex_calculus1_int_symbol,
		'application' => \&tex_calculus1_int_application,
		},
	'partialdiff' => {
		'symbol'      => \&tex_calculus1_partialdiff_symbol,
		'application' => \&tex_calculus1_partialdiff_application,
		},
};

sub tex_calculus1_defint_symbol {
	return ({'fmt' => '\mbox{definite integral}', 'sfmt' => '\mbox{definite integral}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_defint_application {
	my ($node) = @_;
	my $head = $node->{'head'};
	my $args = $node->{'args'};
	my $fmt;

	if( scalar(@$args) != 2 ) {
		&format_die("defint with wrong number of args.\n");
	}

	if( $args->[0]->{'type'} eq 'application' &&
	    $args->[0]->{'head'}->{'type'} eq 'symbol' &&
	    $args->[0]->{'head'}->{'name'} eq 'interval' &&
	    $args->[0]->{'head'}->{'cd'} eq 'interval' ) {
		$fmt = '\int_{' .
			&format_type($args->[0]->{'args'}->[0])->{'sfmt'} .
			'}^{' .
			&format_type($args->[0]->{'args'}->[1])->{'sfmt'} . '}';
	} else {
		$fmt = '\int_{' . &format_type($args->[0])->{'sfmt'} . '}';
	}

	if( $args->[1]->{'type'} eq 'binding' ) {
		$fmt .= &format_type($args->[1])->{'sfmt'};
		$fmt .= '\, d' . &format_type($args->[1]->{'vars'}->[0])->{'sfmt'};
	} else {
		$fmt .= &format_type($args->[1])->{'sfmt'};
	}

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_diff_symbol {
	return ({'fmt' => 'D', 'sfmt' => 'D',
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_diff_application {
	my ($node) = @_;
	my $head = $node->{'head'};
	my $args = $node->{'args'};
	my $fmt;

	if( scalar(@$args) != 1 ) {
		&format_die("diff with wrong number of args.\n");
	}

	$fmt = &tex_bracket_sfmt(&format_type($args->[0]));

	if( $args->[0]->{'type'} eq 'binding' ) {
		$fmt = '\frac{d\ }{d' .
			&format_type($args->[0]->{'vars'}->[0])->{'sfmt'} .
			'}' .
			$fmt;
	} else {
		$fmt = "D " . $fmt;
	}

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

sub tex_calculus1_int_symbol {
	return ({'fmt' => '\mbox{integral}', 'sfmt' => '\mbox{integral}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_int_application {
	my ($node) = @_;
	my $head = $node->{'head'};
	my $args = $node->{'args'};
	my $fmt;

	if( scalar(@$args) != 1 ) {
		&format_die("int with wrong number of args.\n");
	}

	$fmt = '\int ';

	if( $args->[0]->{'type'} eq 'binding' ) {
		$fmt .= &format_type($args->[0])->{'sfmt'};
		$fmt .= '\, d' . &format_type($args->[0]->{'vars'}->[0])->{'sfmt'};
	} else {
		$fmt .= &format_type($args->[0])->{'sfmt'};
	}

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_partialdiff_symbol {
	return ({'fmt' => '{\partial}', 'sfmt' => '{\partial}',
		'lpren' => '', 'rpren' => ''});
}

sub tex_calculus1_partialdiff_application {
	my ($node) = @_;
	my $head = $node->{'head'};
	my $args = $node->{'args'};
	my $bodyfmt;
	my $fmt;
	my $int;
	my $allint;

	if( scalar(@$args) != 2 ) {
		&format_die("partialdiff with wrong number of args.\n");
	}

	&format_type($args->[0]);
	$bodyfmt = &tex_bracket_sfmt(&format_type($args->[1]));

	$allint = 0;
	if( $args->[0]->{'type'} eq 'application' &&
	    $args->[0]->{'head'}->{'type'} eq 'symbol' &&
	    $args->[0]->{'head'}->{'cd'} eq 'list1' &&
	    $args->[0]->{'head'}->{'name'} eq 'list' ) {
		$allint = 1;
		foreach $int ( @{$args->[0]->{'args'}} ) {
			$allint = 0 if( $int->{'type'} ne 'integer' );
			if( $int->{'value'} <= 0 ||
			    $int->{'value'} > scalar(@{$args->[1]->{'vars'}})) {
				die("Invalid index in partialdiff");
			}
		}
	}

	if( $allint && $args->[1]->{'type'} eq 'binding' ) {
		$fmt = '\frac{\partial^{' . scalar(@{$args->[0]->{'args'}}) . '}\ }{';
		foreach $int ( @{$args->[0]->{'args'}} ) {
			$fmt .= '\partial ' .
				&format_type($args->[1]->{'vars'}->[
					$int->{'value'} - 1
				])->{'sfmt'};
		}
		$fmt .=  '}';
	} else {
		$fmt = '\partial_{' . &format_type($args->[0])->{'sfmt'} . '}';
	}

	$fmt .= $bodyfmt;

	return ({'fmt' => $fmt, 'sfmt' => $fmt,
		'lpren' => '\left(', 'rpren' => '\right)'});
}

1;
